
// separate.c
// (c) 2001 Shane Mueller

/*
 This program requires a datafile similarity.dat that contains
 the pairwise similarities of all the words in a set.  The set
 size is predetermined and coded into #define statements
 Initially, the entire set of words is divided up into sets of
 size SETSIZE (12) The program takes two sets of words and
 merges them, and then iteratively picks out words from the set
 that are dissimilar to the set as a whole, until a 'similar'
 and a 'dissimilar' set remain.  Wash, rinse, repeat.
 
 This program is designed to run in the background, regularly saving
 its current state, which can be retrieved at any time or after
 process is killed.  During operation, the files datafile.dat is
 copied to datafile.dat.old, and then a new version of
 datafile.dat is produced.  This allows the program to be exited
 by killing the job (e.g., ctrl-c), without worrying about losing
 the output results.  Consequently, if killed while writing is
 occuring, datafile.dat may not be complete.
 
 If datafile.dat exists when the program is invoked, the program
 will read in that version of lists and continue with the
 procedure from that point.
 
 
 Usually, using the random splicing option is sufficient, although
 some gain  in speed can be achieved if the min or max method is
 used.  However, the non-random methods will quickly settle into a
 stable state, at which point the process can be killed  rerun with
 random splicing to fine-tune the sets.

 Similarity matrices are found in the /sim subdirectory.  Currently,
 seven matrices exist, and a  weighted sum of these matrices is used
 to compute overall similarity.  The matrices include five measures of
 phonological similarity:  onset, nucleus, coda, stress dissimilarity, 
 and "initial  phoneme" similarity, as well as "semantic similarity"
 measured by LSA, and graphemic similarity measured by levenshtein 
 distance. and The weighting is controlled by the global weight array.

 compile with  gcc separate.c -lm -o separate
 */


#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <math.h>
#include <time.h>



#define NUMBEROFWORDS 480
#define NUMBEROFMATRICES 7

inline int min(int a, int b){ return ( (a<b) ? a : b);};
inline int max(int a, int b){ return ( (a<b) ? b : a);};

void mixup();
void splice(int set1, int set2);
void evaluateset(double* setdistances, double*, double*);
void removeitems(int* pool, double*, double*);
void calcdistance(int* pool, int setsize, double*);
double standarddeviation( int);
double  mean( int );
void calcmeandistances();
void initialize();
void inputsimilarity();
void inputdata();
void saveinfo(double *stdev);
void swapids(int* a, int* b);
void printoptions();
int setoption(char * arg1, char* arg2, int);
int processinit();
int processline(int);
int simannealtest(double, double);

char const DataFile[] ="out/datafile.dat";
char const OldDataFile[] ="out/datafile.dat.old";
char const MatrixFileName[8][30]= {
  "matrices/initialmat.dat",
  "matrices/onsetmat.dat",
  "matrices/nucleusmat.dat",
  "matrices/codamat.dat",
  "matrices/stressmat.dat",
  "matrices/lsamat.dat",
  "matrices/leven.dat",
  "matrices/phones.dat"
};


FILE * DistanceFileIn[NUMBEROFMATRICES];      
FILE * WordNames;
FILE * GenericFile;

//Change these to use a different weighting scheme:

double **Weight;
double ***Distance;                                //The distances between items for each contrast
int **   List;                                     //This keep track of the items in each set
double **MeanDistances;                            //The current mean distance of each set for each contrast
char Words[NUMBEROFWORDS][20];
int NumberOfWeightingContrasts;

//int NumberOfRelevantSets; 
int SetSize;
int  NumberOfSets;
int SaveCycle;
double BaselineT;                       //The initial noise parameter
double  Epsilon;                        //The multiplier of BaselineT (every SaveCycles) 
int* MinIndex;                          //The lowest list considered for each eval stat
int* MaxIndex;                          //The highest list considered for each eval stat


//----------------------------------------------
//Global variable that are execution parameters:
//----------------------------------------------


enum EvaluationStatistic {
  esMinMean,
  esMaxMean,
  esMinStd,
  esMaxStd,
  esSplit,
  esNoEval
};

int * EvaluationStat;       //this should be equal to one of the following constants




//=============================================== 
int main(int argc, char *argv[]){
  int i,m;
  int runsim=1;
  double* stddev;
 
  if(argc > 1){
	runsim=0;
	printoptions();
  }; 


   //Process initialization file separate.ini
  if(runsim) runsim = processinit();   //processinit() returns a 1 if reading was successful
  stddev=(double*)malloc(sizeof(double)*NumberOfWeightingContrasts);

  //initialize random number generator
  srand48(time(0));
  
  //read in distance files and
  //read in current word list organization file

  if(runsim){
	initialize(); 

  
	while(1){ 
	  //save info every SaveCycle iterations  

	  for(i=0;i<SaveCycle;i++){
		mixup(); 
	  }; 

	  for(m=0;m<NumberOfWeightingContrasts;m++){
		stddev[m]=standarddeviation(m);
	  };

	  saveinfo(stddev); 
	  BaselineT=BaselineT*Epsilon;
	};  
  };   
  
  free(stddev); 
  return 0;
}


//===============================================
// This selects two sets and performs a splice 
//===============================================
void mixup(){
  int index1;
  int index2;
 
  index1= (lrand48()%NumberOfSets);
 
   do{
	index2=(lrand48()%NumberOfSets);
  }while(index2==index1);
 	
  splice(min(index1,index2),max(index1,index2));   //Merge the two lists and spit them apart

}



//===============================================
//This takes two set indices, merges them, and divides
//them into a similar set and a dissimilar set
//and updates the MeanDistances variable
//===============================================
void splice(int set1, int set2){
  int k,m,success;
  int* pool, *oldpool;
  double *set1dist, *set2dist, *oldset1dist, *oldset2dist;
  double *temp1, *temp2;                                //These will hold the evaluation stats for each contrast

  temp1       = (double*)malloc(sizeof(double)*NumberOfWeightingContrasts);
  temp2       = (double*)malloc(sizeof(double)*NumberOfWeightingContrasts);
  set1dist    = (double*)malloc(sizeof(double)*NumberOfWeightingContrasts);
  set2dist    = (double*)malloc(sizeof(double)*NumberOfWeightingContrasts);
  oldset1dist = (double*)malloc(sizeof(double)*NumberOfWeightingContrasts);
  oldset2dist = (double*)malloc(sizeof(double)*NumberOfWeightingContrasts);


  //This generates two temporary memory regions to do the list splicing with
  pool   =(int*)malloc(2*SetSize*sizeof(int));  
  oldpool=(int*)malloc(2*SetSize*sizeof(int));  



  //This copies target lists into the two pools
  memcpy(pool,List[set1],SetSize*sizeof(int));           //copy stuff into pool
  memcpy(pool+SetSize,List[set2],SetSize*sizeof(int));
  memcpy(oldpool,pool,SetSize*2*sizeof(int));

  calcdistance(pool,         SetSize,oldset1dist);
  calcdistance(pool+SetSize, SetSize,oldset2dist);

  //At this point, various criteria can be used for different ultimate results.
  //These criteria are captured by the enum EvaluationStatistic which is 
  //kept track of by the int EvaluationStat.
  //These stats can differ for different contrasts, so go through each contrast and assign
  //individually.

	//First, evaluate the OLD organization and put the evaluationstats in temp1;
	evaluateset(temp1,oldset1dist, oldset2dist);

  //mix stuff up, find new distances, and assign them to set1dist and set2dist
	removeitems(pool, set1dist, set2dist);                  


	//Now, create the NEW organization:
	// Copy new lists into data structures (temporarily)  
	memcpy(&(List[set1][0]),pool,SetSize*sizeof(int));
	memcpy(&(List[set2][0]),pool+SetSize,SetSize*sizeof(int));

	for(m=0;m<NumberOfWeightingContrasts;m++){
	  MeanDistances[m][set1]=set1dist[m];
	  MeanDistances[m][set2]=set2dist[m];
	};

	//Evaluate the NEW organization, and put the evaluationstats in temp2:
	evaluateset(temp2,set1dist,set2dist);

	
	//Now, see if evaluation was successful and copy everything to
	//the right place if necessary.

	//For each contrast m, we perform a simulated annealing step as well.
	//So, we accept based on how large temp1-temp2 is in proportion to temp1.
	//simannealtest(1, 2) operates so that if 1<=2, 1 is return; if 1>2, 1 is 
	//returned with a probability proportional to (1-2)/1.
	
	success=1;
	for(m=0;m<NumberOfWeightingContrasts;m++){

	  switch(EvaluationStat[m]){
		
	  case esMinMean:
	  case esMinStd:
	    success = success && simannealtest(temp2[m], temp1[m]);
		break;	

	  case esMaxMean:
	  case esMaxStd:
	    success = success && simannealtest(temp1[m], temp2[m]);
	    break;

	  case esSplit:
	    break;
		
	  case esNoEval:
	  default:
	    //success=success;   //Don't do anything.
		break;
		
	  }};

	//If success =1, then every contrast has been successful.
	//If unsuccessful, we should copy everything back the way it originally was
	

	if(!success){

	  memcpy(&(List[set1][0]),oldpool,SetSize*sizeof(int));
	  memcpy(&(List[set2][0]),oldpool+SetSize,SetSize*sizeof(int));

	  for(m=0;m<NumberOfWeightingContrasts;m++){
		MeanDistances[m][set1]=oldset1dist[m];
		MeanDistances[m][set2]=oldset2dist[m];
	  };
	};

	free(pool);
	free(oldpool);
	free(temp1);
	free(temp2);
	free(set1dist);
	free(set2dist);
	free(oldset1dist);
	free(oldset2dist);

}


void evaluateset(double* distances, double* set1dist, double* set2dist){
  int m;
  
  for(m=0;m<NumberOfWeightingContrasts; m++){
	switch(EvaluationStat[m]){


	case esMinMean:       // forcing the mean similarity up/down
	case esMaxMean:
	                           // Copy new results into global data structure for evaluation
                               // If new results didn't succeed, we will copy back.
	  
	  distances[m]=mean(m);
	  
	  break;

	case esMinStd:
	case esMaxStd:               // forcing the stdev up/down             
	                           // Copy new results into global data structure for evaluation
                               // If new results didn't succeed, we will copy back.

	  
	  //find stdev
	  distances[m]= standarddeviation(m);

	  break;


	case esNoEval:
	default:
	  distances[m]=0;
	  //nothing happens here
	};
	


  };
}

//===============================================									
//This removes half the items in pool.  Although it could be
// made smarter (as it once was) so that it tries to divide the
// group in two as well as it can, it currently divides up the group 
// randomly.
//===============================================

void removeitems(int* pool ,double* meandist1, double* meandist2){
  int i,j,k;
  int* divider=pool;
  int argmin=1.0;
  int argmax=0.0;



  
 
	for(i=0;i < 2*SetSize;i++){
	  swapids(pool+i,pool+lrand48()%(SetSize*2-1));
	};

	//Now, just for convenience, put the two sublists in order of their similarities
	calcdistance(pool,        SetSize,meandist1);
	calcdistance(pool+SetSize,SetSize,meandist2);

}

//===============================================
// given two pointers to integers, it swaps their
// contents.
//===============================================
void swapids(int* a, int* b){
  int temp;
  temp=*a;
  *a=*b;
  *b=temp;
}


//===============================================
//this finds the mean similarity (on all contrasts)
//for a given set of words.
//===============================================
void calcdistance(int* pool, int poolsize, double* similarity){
  double sum;
  int i,j,m;
  int e1,e2;

  
  for(m=0;m<NumberOfWeightingContrasts;m++){
	sum=0.0;
	for(i=1;i<poolsize;i++){
	  for(j=0;j<i;j++){
		e1=*(pool+i);
		e2=*(pool+j);
		sum+=Distance[m][e1][e2];
		
	  };
	};

	similarity[m]= sum/(poolsize*(poolsize-1)/2);
  }
}

//===============================================
// calculates standard deviation of distances for 
// contrast index
//===============================================
double standarddeviation(int index){
 
  double x =0.0;
  double xx =0.0;
  int i;

	for(i=MinIndex[index]-1;i<MaxIndex[index]; i++){
	  x +=  MeanDistances[index][i];
	  xx+= pow( MeanDistances[index][i], (double)2);
	};
	i=MaxIndex[index]-MinIndex[index]+1;
	return  sqrt( xx/i - pow((x / i),(double)2));
  
}


double mean(  int index){
  double x=0.0;
  int i;

	for(i=MinIndex[index]-1; i < MaxIndex[index]; i++){
	  x+=MeanDistances[index][i];
	};
	return  x/(MaxIndex[index]-MinIndex[index]+1);
	
}




//===============================================
//simannealtest(1, 2) operates so that if 1<=2, 1 is return; if 1>2, 1 is 
//returned with a probability proportional to (1-2)/1.
//===============================================

int simannealtest(double val1, double val2){
  if(val1 <= val2){
    // printf("+");
    return 1;
  }
  else{
    if(drand48() < exp(-((val1-val2)/val1)/BaselineT)){
      //  printf("%f %f %f\n", val1, val2 ,exp(-((val1-val2)/val1)/BaselineT)) ;
      // printf("*");
      return 1;
    }
    else{
      // printf("-");
      return 0;
    };
  };
}


//===============================================
// calculates the distances of the sets
//===============================================
void calcmeandistances(){
  int i,m;
  double * tempmean;

 tempmean=(double*)malloc(sizeof(double)*NumberOfWeightingContrasts);
  for(i=0;i<NumberOfSets;i++){
	calcdistance(&(List[i][0]),SetSize, tempmean);
	for(m=0;m<NumberOfWeightingContrasts;m++){
	  MeanDistances[m][i]=tempmean[m];
	};
  };
  free(tempmean);

}


//=============================================================================================
//=============================================================================================
//                                      I/O functions
//=============================================================================================
//=============================================================================================


void initialize(){
 
  printf("Initializing...Reading In Data\n");
  inputsimilarity();
  inputdata();
  calcmeandistances();
    
}



//===============================================
// This reads in the similarity matrix
// It has the capability to combine the similarity matrices in
// any possible way. It does this for each of the NumberOfWeightingContrasts matrices
//===============================================


void inputsimilarity(){
  int i,j,k,m; 
  char line[100];
  char line2[10];
  double temp;
  //Initialize Distance**,
  //Distance is Distance[NumberOfWeightingContrasts][NUMBEROFWORDS][NUMBEROFWORDS]
  Distance=(double***)malloc(sizeof(double**)*NumberOfWeightingContrasts);
  //MeanDistances[NumberOfWeightingContrasts][NumberOfSets]
  MeanDistances=(double**)malloc(sizeof(double*) * NumberOfWeightingContrasts);

 

  for(m=0;m<NumberOfWeightingContrasts; m++){
	Distance[m]=(double**)malloc(sizeof(double*)*NUMBEROFWORDS);
	MeanDistances[m]=(double*)malloc(sizeof(double)*NumberOfSets);


	for(i=0; i<NUMBEROFWORDS; i++){
	  Distance[m][i]=(double*)malloc(sizeof(double)*NUMBEROFWORDS);
	  memset(Distance[m][i],0,sizeof(double)*NUMBEROFWORDS);
	};
  };


  	//open the files
  for(k=0;k<NUMBEROFMATRICES;k++){
	DistanceFileIn[k] = fopen(MatrixFileName[k], "r" );
  };

  WordNames=fopen(MatrixFileName[NUMBEROFMATRICES],"r");  //The file containing word names
 
 

	for(i=0;i<NUMBEROFWORDS;i++){
	  for(j=0;j<NUMBEROFWORDS;j++){
		for(k=0;k<NUMBEROFMATRICES;k++){
	
		  fscanf(DistanceFileIn[k],"%s ", line);           //enter a weighted sum into each matrix entry
		  temp=atof(line);
		  for(m=0;m<NumberOfWeightingContrasts;m++){	  
			Distance[m][i][j]+= temp * Weight[m][k];
		  };
		};
	  };
	  printf(".");
	  fflush(stdout);
		
	};  
	printf("\n");
	
  //Now, get each of the word names
  for(i=0;i<NUMBEROFWORDS;i++){
	fgets(line,100,WordNames);
	sscanf(line,"%s %s ",line2,Words[i]);
  };

  //close the files

  for(i=0;i<NUMBEROFMATRICES;i++){
	fclose(DistanceFileIn[i]);
  };
  fclose(WordNames);
} 



//===============================================
// This reads in the current list configuration 
// (if there is one) or creates its own
// configuration (if there isn't one).
//===============================================

void inputdata(){
  int i,j,k;
  char line[100];
  int temp;
  int fileok;

  // printf("Entering inputdata\n");
 if( (GenericFile=fopen(DataFile,"r"))==NULL){
   //No previous data file exists

   k=0;
   printf("No List organization file exists.  Generating.\n");
   for(i=0;i<NumberOfSets;i++){
	  for(j=0;j<SetSize; j++,k++){
			List[i][j]=k;
	  };
	};
  }
  else{
	//The input file still might be a dud, so check for feof()
	printf("Reading in old list organization file.\n");
	fileok=0;
	for(i=0;i<NumberOfSets;i++){
	  for(j=0;j<SetSize;j++){
	
		fileok=feof(GenericFile);
		fscanf(GenericFile,"%s ",line);          
		temp=atoi(line);
		List[i][j]=temp;
	  }
	};

	
	fclose(GenericFile);
	//If fileok is 1, the file ended before we got done reading.
	if(fileok==1){
	  printf("Old Data File Corrupt:");
	  getchar();

	  remove(DataFile);
	  inputdata();
	};
  };
}




//===============================================
// This copies current output file to output.old
// and output a new configuration file in its place
//===============================================
void saveinfo(double * stdev){
  int i,j,m;
  int printit;
  double *grandmean;
  grandmean=(double*)malloc(sizeof(double)*NumberOfWeightingContrasts);

  //Calculate the grand mean for each dimension
  for(m=0;m<NumberOfWeightingContrasts;m++){
	grandmean[m] = mean(m);
  };

  if(GenericFile=fopen(OldDataFile,"r")){
	fclose(GenericFile);
	remove(OldDataFile);       //get rid of old datafile
  };

  if(GenericFile=fopen(DataFile,"r")){
	fclose(GenericFile);
	rename(DataFile,OldDataFile); //now datafile shouldn't exist
  };


  GenericFile = fopen(DataFile, "w" );
	for(i=0;i<NumberOfSets;i++){
	  for(j=0;j<SetSize;j++){
		fprintf(GenericFile,"%d ",List[i][j]);
	  };
	  fprintf(GenericFile,"\n");
	};

	fprintf(GenericFile,"----------------------\n");
	fprintf(GenericFile,"Beyond this point, data is not read.\n");



	//Print out some interesting stats:

	   fprintf(GenericFile,"Number of Words: %d\n", NUMBEROFWORDS);	
	   fprintf(GenericFile,"Set Size: %d\n",SetSize);
	   fprintf(GenericFile,"Number Of Sets: %d\n", NumberOfSets);
	   fprintf(GenericFile,"Number of Contrasts: %d\n",NumberOfWeightingContrasts);
	   fprintf(GenericFile,"Simulated Annealing Noise Parameter: %f\n", BaselineT);

	for(m=0;m<NumberOfWeightingContrasts;m++){
	  fprintf(GenericFile,"Contrast %d: ",m+1);

	  for(j=0;j<NUMBEROFMATRICES;j++){
		fprintf(GenericFile, "%f  ", Weight[m][j]);
	  };
	  switch(EvaluationStat[m]){
	  case esMinMean:
		fprintf(GenericFile, "\nEvaluation Statistic: Minimize Mean\n");
		break;
	  case esMaxMean:
		fprintf(GenericFile, "\nEvaluation Statistic: Maximize Mean\n");
		break;
	  case esMinStd:
		fprintf(GenericFile, "\nEvaluation Statistic: Minimize Standard Deviation\n");
		break;
	  case esMaxStd:
		fprintf(GenericFile, "\nEvaluation Statistic: Maximize Standard Deviation\n");
		break;

	  case esNoEval:
		fprintf(GenericFile, "\nEvaluation Statistic: No Criteria\n");
		break;
	  default:
		fprintf(GenericFile, "\nEvaluation Statistic: Unknown\n");
		break;};

	  fprintf(GenericFile,"Relevant Sets:  %d to %d\n", MinIndex[m], MaxIndex[m]);

	  fprintf(GenericFile,"Mean:  %f  Standard Deviation:  %f  \n",grandmean[m], stdev[m]);
	  printf("%1.4f %1.4f | ",grandmean[m], stdev[m]);
	  fprintf(GenericFile,"--------------------------\n");
	};
	printf(" SA: %f\n",BaselineT);
	for(i=0;i<NumberOfSets;i++){
	  

	  //Print a line if this is a dividing set
	  printit=0;
	  for(m=0;m<NumberOfWeightingContrasts;m++){
	    if((i+10000)==MinIndex[m] || (i)==MaxIndex[m]) printit=1; 
	  };
	  if(printit) fprintf(GenericFile, "----------------------------------------------\n");
	  

	  for(m=0;m<NumberOfWeightingContrasts;m++){
		fprintf(GenericFile, "%f  ",MeanDistances[m][i]);  
	  };
	  fprintf(GenericFile,": ");
	  for(j=0;j<SetSize;j++){
	 	fprintf(GenericFile,"%s ", Words[List[i][j]]);
	  };
	  fprintf(GenericFile,"\n");
	  
	};
	
	fclose(GenericFile);
	free(grandmean);
}




//===============================================
// This copies current output file to output.old
// and output a new configuration file in its place
//===============================================

void printoptions(){
  printf("\n-------------------------------------------------------------\n");
  printf("The SEPARATE utility.  Copyright 2001 by Shane T. Mueller\n");
  printf("www-personal.umich.edu/~smueller          smueller@umich.edu\n");
  printf("-------------------------------------------------------------\n");
  printf("This utility uses the words from the Toronto Noun Pool \n");
  printf("and attempts to cluster them into groups using a genetic-\n");
  printf("algorithm-like process.                                  \n");
  printf("The program's parameters are controlled by a file        \n");
  printf("separate.ini. This file should contain the following lines:\n");
  printf("<SET SIZE>\n");
  printf("<NUMBER OF RELEVANT SETS>\n");
  printf("<NUMBER OF CYCLES BETWEEN DISPLAY/SAVE>\n");
  printf("<WEIGHTING CONTRAST> -es <evalution statistic>\n");
  printf("Multiple weighting contrasts/evalutation statistics can be\n");
  printf("used by including multiple lines.\n");
  printf("Weighting contrasts take the form of 7 numbers that describe\n");
  printf("how to combine the seven dimensions of the similarity profile\n");
  printf("The evauluation statistic is one of:\n");
  printf("  -es minmean     Splice decreases mean contrast.\n");
  printf("  -es maxmean     Splice increases mean contrast.\n");
  printf("  -es minstd      Splice decreases std. dev. of contrast.\n");
  printf("  -es maxstd      Splice increases std. dev. of contrasts.\n");
  printf("  -es noeval      No dimension not evaluated (for monitoring\n");
  printf("------------------------------------------------------------\n");
};

int setoption(char * arg1, char* arg2, int j){
 
  if(!strcmp(arg1,"-es")){
	if(!strcmp(arg2,"minmean"))       EvaluationStat[j]=esMinMean;
	else if(!strcmp(arg2,"maxmean"))  EvaluationStat[j]=esMaxMean;
	else if(!strcmp(arg2, "minstd"))  EvaluationStat[j]=esMinStd;
	else if(!strcmp(arg2, "maxstd"))  EvaluationStat[j]=esMaxStd;
	else if(!strcmp(arg2, "noeval"))      EvaluationStat[j]=esNoEval;
	else   EvaluationStat[j]=esNoEval;
     return 1;
  }
   else{
      printf("Incorrect Options.  Using default values\n");
      EvaluationStat[j]=esNoEval;
      return 0;
  }
   
};


//This function reads in separate.ini (if there is one)
//and sets up the appropriate parameters for the procedure
//based on the lines in the that file.
int processinit()
{
   char line[150];
   int  j, error;
 
   if( GenericFile=fopen("separate.ini","r"))
	 {
	   //Now, read in the first four lines.  These are (with generic initiating variables):

	   SetSize=10;                                   //Size of set.  Must be a divisor of NUMBEROFWORDS
	   NumberOfSets=NUMBEROFWORDS/SetSize;           //Total number of sets--NOT IN FILE
	   SaveCycle =1000;                              //Number of cycles between saves/displays


	   //Line by line, read the input file
	   
	   //first line is list size
	   fscanf(GenericFile,"%s",line);
	   SetSize=atoi(line);
	   NumberOfSets=NUMBEROFWORDS/SetSize;

	   //Second line is the number of cycles between saves/displays
	   fscanf(GenericFile,"%s",line);
	   SaveCycle=atoi(line);

	
	   //Third line is number of weighting contrasts
	   fscanf(GenericFile,"%s",line);
	   NumberOfWeightingContrasts=atoi(line);
	
	   //Fourth line is the simulated annealing noise parameters
	   fscanf(GenericFile,"%s",line);
 	   BaselineT = atof(line);
 	   fscanf(GenericFile,"%s", line);
 	   Epsilon = atof(line);

	   /********************************************************
	    *** Now, we know the right sizes for things, so initiate the
            *** arrays to be of the right size.
            ********************************************************/


	   //Now, initiate List to be the right size.
	   List=(int**)malloc(sizeof(int*)*NumberOfSets);
	   for(j=0;j<NumberOfSets;j++){
		 List[j]=(int*)malloc(sizeof(int)*SetSize);
	   };

	   
	   //initiate the weight matrix
	   Weight=(double**)malloc(sizeof(double*)*NumberOfWeightingContrasts);
	   for( j=0; j<NumberOfWeightingContrasts; j++){
	     Weight[j]=(double*)malloc(sizeof(double)*NUMBEROFMATRICES);
	   };
	   

	   //Initiate MinIndex and MaxIndex and EvaluationStat
	   MinIndex=(int*)malloc(sizeof(int)*NumberOfWeightingContrasts);
	   MaxIndex=(int*)malloc(sizeof(int)*NumberOfWeightingContrasts);
	   EvaluationStat = (int*)malloc(sizeof(int) * NumberOfWeightingContrasts);
	   

	   
	   error=1;
	   for(j=0; j<NumberOfWeightingContrasts; j++){
	     error *= processline(j);
	   };

	   
	 }            
   else
     {//File doesn't exist
       printf("File separate.ini doesn't exist.\n");
		error = 0;
	 };
   
   if(!error) printoptions();
   fclose(GenericFile);
   return error;
   }

//This processes a single line of the separate.ini file
int processline(j){
  
  int i;
  char tmp[10];
  char flag[5];
  char argument[10];


  printf("Reading Weight Vector:  ");
  //read in the weights
  for(i=0;i<NUMBEROFMATRICES;i++)
	{
	  fscanf(GenericFile,"%s ",  tmp);
	  Weight[j][i]=atof(tmp);
	  printf("%f ", Weight[j][i]);
	};

  printf("\n");
  
  //Send the next to tokens to setoption
  fscanf(GenericFile,"%s %s", flag, argument);
  printf("Option: %s %s\n", flag, argument);
  i=setoption(flag, argument, j);

  //The last two pieces indicate the first and last sets that matter.
  fscanf(GenericFile,"%s", tmp);
  MinIndex[j]=atoi(tmp);
  fscanf(GenericFile,"%s", tmp);
  MaxIndex[j]=atoi(tmp);
  
  printf(" From sets  %d to %d\n-----------\n",MinIndex[j],MaxIndex[j]);
  return i;
};
